﻿Imports ThinkGeo.MapSuite.Core
Imports ThinkGeo.MapSuite.DesktopEdition
Imports System.Collections.ObjectModel
Public Class Form1
   Private moLine As LineShape
   Private moPolygon As PolygonShape
   Private moMap As WinformsMap
   Private moShapefile As MyShapefile
   Private moGeoShapefile As MyShapefile
   Private moInMemoryFeatureLayer As New InMemoryFeatureLayer
   Private WithEvents moTrackLayer As TrackInteractiveOverlay
   Private WithEvents moEditOverlay As MyEditOverlay
   Private msKey As String
   Private moEditFeatures As New Collection(Of Feature)
   Public Const cdMICRON As Double = 0.000001
   Private msInstructions As String =
      "1. Select a red feature by clicking with the mouse." & ControlChars.CrLf &
      "2. Drag a node to edit a feature, the transaction will remain uncommitted." & ControlChars.CrLf &
      "3. Now click off the feature or try to select a different feature." & ControlChars.CrLf &
      "4. Notice the ArgumentOutOfRangeException exception."

   Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
      Dim oLayerOverlay As New LayerOverlay
      Dim oSelectionOverlay As New LayerOverlay

      Me.TextBox1.Text = msInstructions
      moMap = WinformsMap1
      moMap.MapUnit = GeographyUnit.Meter
      moMap.MinimumScale = 0
      ShapeFileFeatureLayer.Rebuild("..\..\Data\TestPoly.shp")
      moShapefile = New MyShapefile("..\..\Data\TestPoly.shp", ShapeFileReadWriteMode.ReadWrite)

      'Dim oProj4 As New ManagedProj4Projection(Proj4Projection.GetEsriParametersString(26916), Proj4Projection.GetEsriParametersString(4269))
      'moShapefile.FeatureSource.Projection = oProj4


      moShapefile.Name = "TestPoly"
      oLayerOverlay.Layers.Add("Shapefile", moShapefile)

      Dim oAreaStyle As AreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.SimpleColors.Transparent, GeoColor.SimpleColors.Blue, 1)
      moShapefile.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(oAreaStyle)
      moShapefile.AreaSymbol = oAreaStyle

      Dim oTextStyle As TextStyle = TextStyles.Country1("Name")
      oTextStyle.DuplicateRule = LabelDuplicateRule.UnlimitedDuplicateLabels
      oTextStyle.OverlappingRule = LabelOverlappingRule.AllowOverlapping
      oTextStyle.LabelAllPolygonParts = True
      moShapefile.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(oTextStyle)
      moShapefile.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20


      moGeoShapefile = New MyShapefile("..\..\Data\GCS1983Polys.shp", ShapeFileReadWriteMode.ReadWrite)
      oAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.SimpleColors.Transparent, GeoColor.SimpleColors.Red, 1)
      moGeoShapefile.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(oAreaStyle)
      moGeoShapefile.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20
      moGeoShapefile.AreaSymbol = oAreaStyle
      Dim oProj4 As New ManagedProj4Projection(Proj4Projection.GetEsriParametersString(4269), Proj4Projection.GetEsriParametersString(26916))
      moGeoShapefile.FeatureSource.Projection = oProj4

      oLayerOverlay.Layers.Add("GeoShapefile", moGeoShapefile)

      moShapefile.TextSymbol = oTextStyle

      WinformsMap1.Overlays.Add(oLayerOverlay)

      moEditOverlay = New MyEditOverlay
      WinformsMap1.EditOverlay = moEditOverlay
      With moEditOverlay
         .CanAddVertex = False
         .CanDrag = False
         .CanRemoveVertex = False
         .CanReshape = True
         .CanResize = False
         .CanRotate = False
         .EditLayer = moGeoShapefile
         .Map = WinformsMap1
      End With

      moTrackLayer = WinformsMap1.TrackOverlay

      moGeoShapefile.Open()
      WinformsMap1.CurrentExtent = moGeoShapefile.GetBoundingBox
      WinformsMap1.Refresh()
   End Sub

   Private Sub WinformsMap1_KeyUp(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles WinformsMap1.KeyUp
      Dim oSelectionLayer As InMemoryFeatureLayer = WinformsMap1.EditOverlay.EditShapesLayer

      If oSelectionLayer.InternalFeatures.Count > 0 Then
         If moShapefile.IsOpen Then
            moShapefile.Open()
         End If
         moShapefile.EditTools.BeginTransaction()
         For Each oFeature As Feature In oSelectionLayer.InternalFeatures
            moShapefile.EditTools.Delete(oFeature.Id)
         Next oFeature
         moShapefile.EditTools.CommitTransaction()
         moShapefile.Close()
         ShapeFileFeatureLayer.BuildIndexFile(moShapefile.ShapePathFileName)
         oSelectionLayer.InternalFeatures.Clear()
         WinformsMap1.Refresh()
      End If

   End Sub


   Private Sub WinformsMap1_MapClick(ByVal sender As Object, ByVal e As ThinkGeo.MapSuite.DesktopEdition.MapClickWinformsMapEventArgs) Handles WinformsMap1.MapClick
      Dim oFeatures As Collection(Of Feature)
      If e.MouseButton = MapMouseButton.Left Then

         If msKey <> String.Empty Then
            With WinformsMap1.EditOverlay.EditShapesLayer
               .Open()
               .EditTools.IsTransactionLive = True
               .EditTools.BeginTransaction()
               .EditTools.Delete(msKey)
               .EditTools.CommitTransaction()
            End With
         End If

         With CType(WinformsMap1.EditOverlay, MyEditOverlay).EditLayer
            If Not .IsOpen Then
               .Open()
            End If
            oFeatures = .QueryTools.GetFeaturesWithinDistanceOf(
                     New PointShape(e.WorldX, e.WorldY),
                     moMap.MapUnit,
                     DistanceUnit.Meter,
                     0.01,
                     ReturningColumnsType.AllColumns)


            If oFeatures.Count > 0 Then
               msKey = oFeatures(0).Id
               With WinformsMap1.EditOverlay.EditShapesLayer
                  .Open()
                  .EditTools.IsTransactionLive = True
                  .EditTools.BeginTransaction()
                  .EditTools.Add(oFeatures(0))
                  .EditTools.CommitTransaction()
               End With
            End If
            .Close()
            moEditOverlay.CalculateAllControlPoints()
         End With


         WinformsMap1.Refresh()
      End If
   End Sub

   Private Sub mCommitTransaction()
      If Not moShapefile.IsOpen Then
         moShapefile.Open()
      End If

      If moShapefile.EditTools.IsInTransaction Then
         moShapefile.EditTools.CommitTransaction()
         moShapefile.Close()
         WinformsMap1.Refresh()
      End If

   End Sub
End Class
